<?php

/*********************************************************************

 メディア文章表示プラグイン (2012/10/26)

 Copyright(C) 2009-2012 freo.jp

*********************************************************************/

/* メイン処理 */
function freo_page_media_document()
{
	global $freo;

	if (isset($_GET['file']) and preg_match('/^[\w\-\/\.]+$/', $_GET['file']) and !preg_match('/\.\./', $_GET['file'])) {
		if (preg_match('/' . FREO_ASCII_FILE . '/i', $_GET['file'])) {
			//ファイル情報取得
			list($file_width, $file_height, $file_size) = freo_file(FREO_FILE_DIR . 'medias/' . $_GET['file']);

			$information = array();
			$text        = null;
			$flag        = false;

			if ($fp = fopen(FREO_FILE_DIR . 'medias/' . $_GET['file'], 'r')) {
				while ($line = fgets($fp)) {
					$line = freo_trim($line);

					if (preg_match('/' . FREO_PLUGIN_MEDIA_DOCUMENT_INFORMATION_START . '/', $line)) {
						$flag = true;
					} elseif (!$flag) {
						$text .= $line . "\n";
					} elseif (preg_match('/' . FREO_PLUGIN_MEDIA_DOCUMENT_INFORMATION_END . '/', $line)) {
						$flag = false;
					}

					if ($flag and preg_match('/([^:]+):(.+)/', $line, $matches)) {
						$information[$matches[1]] = $matches[2];
					}
				}

				fclose($fp);
			}

			$text = freo_trim($text);

			//表示形式調整
			if (!preg_match('/\.(html|htm||php)$/i', $_GET['file'])) {
				if (empty($information['html']) or $information['html'] != 'enabled') {
					$text = htmlspecialchars($text, ENT_QUOTES);
				}

				$text = '<p>' . nl2br($text) . '</p>';
			}

			//データ割当
			$freo->smarty->assign(array(
				'token'                      => freo_token('create'),
				'media_document_name'        => basename($_GET['file']),
				'media_document_width'       => $file_width,
				'media_document_height'      => $file_height,
				'media_document_size'        => $file_size,
				'media_document_text'        => $text,
				'media_document_information' => $information
			));

			//データ出力
			freo_output('plugins/media_document/file.html');
		} else {
			//リダイレクト
			freo_redirect($freo->core['http_url'] . FREO_FILE_DIR . 'medias/' . $_GET['file']);
		}
	} else {
		//置換用ディレクトリ名取得
		$media_document_names = array();
		$directory_names      = explode("\n", $freo->config['plugin']['media_document']['directory_names']);

		foreach ($directory_names as $directory_name) {
			if (!$directory_name) {
				continue;
			}

			list($id, $text) = explode(',', $directory_name, 2);

			$media_document_names[$id] = $text;
		}

		//メディア取得
		$media_documents = freo_page_media_document_get_medias(FREO_FILE_DIR . 'medias/');

		//データ割当
		$freo->smarty->assign(array(
			'token'                => freo_token('create'),
			'media_documents'      => $media_documents,
			'media_document_names' => $media_document_names
		));
	}

	return;
}

/* メディア取得 */
function freo_page_media_document_get_medias($path)
{
	global $freo;

	$files = array();

	if (!file_exists($path)) {
		return $files;
	}

	if ($dir = scandir($path)) {
		natcasesort($dir);

		if ($freo->config['plugin']['media_document']['order'] == 'name_desc') {
			$dir = array_reverse($dir);
		}

		$tmp_directories = array();
		$tmp_files       = array();

		foreach ($dir as $data) {
			if ($data == '.' or $data == '..') {
				continue;
			}

			if (is_dir($path . $data)) {
				$tmp_directories[] = $data;
			} elseif (is_file($path . $data)) {
				$tmp_files[] = $data;
			}
		}

		$dir = array_merge($tmp_files, $tmp_directories);
	}

	foreach ($dir as $data) {
		if ($data == '.' or $data == '..') {
			continue;
		}
		if (is_dir($path . $data)) {
			$results = freo_page_media_document_get_medias($path . $data . '/');

			if (!empty($results)) {
				$files = array_merge($files, $results);
			}

			if (preg_match('/^' . preg_quote(FREO_FILE_DIR . 'medias/', '/') . '(.+)$/', $path . $data, $matches)) {
				$directory = $matches[1];
			} else {
				$directory = null;
			}

			$files[$path . $data . '/'] = array(
				'id'          => $path . $data . '/',
				'path'        => $path,
				'name'        => $data,
				'type'        => 'directory',
				'directory'   => $directory,
				'width'       => 0,
				'height'      => 0,
				'size'        => 0,
				'information' => array()
			);
		} elseif (is_file($path . $data)) {
			list($file_width, $file_height, $file_size) = freo_file($path . $data);

			$information = array();
			$flag        = false;

			if ($fp = fopen($path . $data, 'r')) {
				while ($line = fgets($fp)) {
					$line = freo_trim($line);

					if (preg_match('/' . FREO_PLUGIN_MEDIA_DOCUMENT_INFORMATION_START . '/', $line)) {
						$flag = true;
					} elseif (preg_match('/' . FREO_PLUGIN_MEDIA_DOCUMENT_INFORMATION_END . '/', $line)) {
						$flag = false;
					}

					if ($flag and preg_match('/([^:]+):(.+)/', $line, $matches)) {
						$information[$matches[1]] = $matches[2];
					}
				}

				fclose($fp);
			}

			if (preg_match('/^' . preg_quote(FREO_FILE_DIR . 'medias/', '/') . '(.+)$/', $path . $data, $matches)) {
				$file = $matches[1];
			} else {
				$file = null;
			}

			$files[$path . $data] = array(
				'id'          => $path . $data,
				'path'        => $path,
				'name'        => $data,
				'type'        => 'file',
				'file'        => $file,
				'width'       => $file_width,
				'height'      => $file_height,
				'size'        => $file_size,
				'information' => $information
			);
		}
	}

	return $files;
}

?>
